# Generated by Django 4.2.14 on 2024-08-16 21:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_contacts', '0004_standingfilter'),
    ]

    operations = [
        migrations.AddField(
            model_name='standingfilter',
            name='check_type',
            field=models.CharField(choices=[('any', 'At Least One Character'), ('all', 'All Characters'), ('no', 'No Character')], default='any', max_length=3),
        ),
        migrations.AddField(
            model_name='standingfilter',
            name='only_main',
            field=models.BooleanField(default=False, help_text='Only consider the main character of the user'),
        ),
    ]
