# Generated by Django 4.2.13 on 2024-05-30 19:29

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('esi', '0012_fix_token_type_choices'),
        ('aa_contacts', '0002_alter_general_options_alter_alliancecontact_labels_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='corporationcontact',
            name='is_watched',
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name='CorporationToken',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_update', models.DateTimeField(default=django.utils.timezone.now)),
                ('corporation', models.OneToOneField(on_delete=django.db.models.deletion.RESTRICT, related_name='+', to='eveonline.evecorporationinfo')),
                ('token', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='esi.token')),
            ],
            options={
                'default_permissions': (),
            },
        ),
    ]
